;****************************************************************************************************
;* T24 Lastgeregelter DCC dekoder 8MHz int RC                                                050309 *
;*                                                                                                  *
;*                                   ************   ***********                                     *
;*                                1  * VCC       ***      GND * 14                                  *
;*        Licht vorwrts          2  * PB0                PA0 * 13        Generator ADC             *
;*        Licht rckwrts         3  * PB1                PA1 * 12        F1                        *
;*        Reset                   4  * Reset              PA2 * 11        F2                        *
;*        DCC Eingang             5  * PB2/INT0           PA3 * 10        F3                        *
;*                                6  * PA7                PA4 *  9        F4                        *
;*        Motor PWM               7  * PA6                PA5 * 08        Motor Richtung            *
;*                                   **************************                                     *
;*                                                                                                  *
;* DCC Empfangsschleife von Maddax (http://www.mikrocontroller.net/topic/31397#new)                 *
;* alles andere von Toralf Wilhelm private Nutzung erwnscht,                                       *
;* gewerbliche Nutzung erfordert zwingend meine Zustimmung!                                         *
;* Folgende AVR Hardware wird verwendet: 8MHz RC Osc.,8Bit Timer im CTC Modus, 16Bit Timer im Fast  *
;* PWM Modus, 8Bit ADC, 94Byte EEprom, 46Byte Sram                                                  *
;* Wert fr Oscillator Calibration Register in EEPROMZelle 100 ablegen!                             *
;* Folgende Funktionen sind programmiert:                                                           *
;* - kurze und lange Adressen                                                                       *
;* - 14,28 und 128 Fahrstufen ber CV2/5 oder bei 14/28FS ber Kennlinie in CV67-94                 *
;* - Anfahr/Bremsverzgerung in CV3/4                                                               *
;* - Motor PWM Frequenz mit: 30Hz,120Hz,500Hz,16kHz oder 32kHz in CV9                               *
;* - analogBetrieb > fehlt noch (Hardware abhngig) kommt noch <                                    *
;* - optionales sichern der letzten FS+Richtung im EEprom (ist keine DCC Norm)                      *
;* - Fahrlicht Richtungsabhngig mit 80Hz in 10 Stufen Dimmbar in CV50                              *
;* - Funktionsgruppe1 ein/aus direkt an Ausgnge F1-F4                                              *
;* - Lastreglung mit Schrittweite, max. Korrekturweite, mindest Abweichung in CV52,53,54            *
;* - Kickstart zeit und Spannung in CV 65,66                                                        *
;* - Kennline fr 14/28 FS in CV67-94                                                               *
;* - Programmierung ber ServiceMode momentan nur CVByte schreiben                                  *
;* - Programmierumg "auf den Hauptgleis" (POM) momentan nur CVByte schreiben                        *
;****************************************************************************************************

.include "tn24def.inc"

;********************************
;* FUSE info: fr 8MHz int RC   *
;*                              *
;* SELFPRGEN:     nein(1)       *
;* RSTDISBL:      nein(1)       *      ;RESET Pin RESET Funktion ausgeschaltet
;* DWEN:          nein(1)       *      ;Debug eingeschaltet
;* WDTON:         nein(1)       *      ;WDT beim Start an
;* SPIEN:           ja(0)       *      ;serial programmieren erlaubt
;* EESAVE:        nein(1)       *      ;eeprom wird bei chiperase auch gelscht
;* BODLEVEL:         100        *      ;BOD bei ca 4,3V VCC
;* SUT1:            ja(0)       *      ;SUT 00 = BOD ein
;* SUT0:            ja(0)       *
;* CKDIV8:        nein(1)       *      ;kein CK vorteiler
;* CKOUT:         nein(1)       *      ;PB2 CK out
;* CKSEL3:          ja(0)       *      ;CKSEL 0010 = int 8 MHz RC
;* CKSEL2:        nein(1)       *
;* CKSEL1:          ja(0)       *
;* CKSEL0:          ja(0)       *
;* Fuse HighByte: $D4           *
;* Fuse LowByte:  $C2           *
;********************************

;************************************************************************************************
;*                         Konstantendef.                                                       *
;************************************************************************************************

;Ports
.equ    MotR          =    5           ; PA5 Motor Richtung Ausgang
.equ    MotPWM        =    6           ; PA6 Motor PWM Ausgang
.equ    LichtV        =    0           ; PB0 Licht vorwrts
.equ    LichtR        =    1           ; PB1 Licht rckwrts
.equ    F1            =    1           ; PA1 Funktionsausgang 1
.equ    F2            =    2           ; PA2 Funktionsausgang 2
.equ    F3            =    3           ; PA3 Funktionsausgang 3
.equ    F4            =    4           ; PA4 Funktionsausgang 4

.equ    fosc          =     8000000    ; Systemtakt in Hz

;Flagregister
.equ    resetbit      =    0           ;
.equ    frei1         =    1           ;
.equ    frei2         =    2           ;
.equ    kHzPWM        =    3           ; 1 = Motor PWM 16 oder 32 kHz
.equ    bit           =    4           ; "das" neue Bit
.equ    neuesbit      =    5           ; es ist ein neues DCC Bit eingetroffen
.equ    PWM9Bit       =    6           ; 1 = PWM luft mit 9-Bit -> 16KHz
.equ    keinDCC       =    7           ; fr analogbetrieb noch nicht aktiv

;DCCregister
.equ    Richtung_inv  =    0           ; 1= Fahrtrichtung invertiert
.equ    Fahrstufen    =    1           ; 1= 28/128FS 0= 14FS
.equ    analog        =    2           ; 0= nur DCC
.equ    Vmerken       =    3           ; 1= merke letzten Fahrbefehl im EErpom
.equ    nutzeKennl    =    4           ; 1= nutze Kennlinie
.equ    erwAdresse    =    5           ; 1= nutze lange Adresse
.equ    FL            =    6           ; Licht an
.equ    Richtung      =    7           ; Fahrtrichtung wenn nicht inv. 1=vor 0=zurck

;************************************************************************************************
;*                         Registerdef.                                                         *
;************************************************************************************************

.def    Adresse       =    r0          ; eigene Adresse bzw bei langen Adressen der high wert
.def    V_IST         =    r1          ; momentane Geschwindigkeit
.def    V_SOLL        =    r2          ; Soll Geschwindigkeit
.def    intsreg       =    r3          ; Kopie sreg im Interrupt (zum beschleunigen vom interrupt)
.def    FSzaehler     =    r4          ; Fahrstufenzaehler zum dec/inc der Fahrstufen
.def    ProgTest      =    r5          ; zum vergleichen beim CV schreiben
.def    EMK_soll      =    r6          ; Sollwert fr Lastreglung
.def    EMK_pos       =    r7          ; aktuelle regelposition (Abweichung vom sollwert fr max Begrenzung)
.def    LregMin       =    r8          ; Mindestabweichung bevor Lastreglung einsetzt
.def    Dimmer        =    r9          ; Licht - Dimmer
.def    byteA         =    r10         ; Empfangsbytespeicher
.def    byteB         =    r11
.def    byteC         =    r12
.def    byteD         =    r13
.def    byteE         =    r14
.def    byteF         =    r15
.def    bitzaehler    =    r16         ; Bitzaehler beim speichern
.def    w             =    r17         ; Arbeitsregister
.def    flag          =    r18         ; Flagregister
                                       ;    Bit0 = 1 -> resetbit
                                       ;    Bit1 = 1 -> frei
                                       ;    Bit2 = 1 -> frei
                                       ;    Bit3 = 1 -> Vsoll im EEProm merken
                                       ;    Bit4 = 1 -> das neue bit
                                       ;    Bit5 = 1 -> neues bit da
                                       ;    Bit6 = 1 -> PWM Pin ein im ctc interrupt
                                       ;    Bit7 = 1 -> lnger als 10000us keine bits eingetroffen
.def    DCCReg        =    r19         ; DCC Register fr Funktionen und Fahrtrichtung
                                       ;    Bit7 = 1 -> 28 Fahrstufen (0 = 14 Fahrstufen)
                                       ;    Bit6 = 1 -> Fahrtrichtung reversiert
                                       ;    Bit5 Fahrtrichtung 1=vorwrts, 0=rckwrts
                                       ;   Bit4
                                       ;    Bit3
                                       ;    Bit2
                                       ;    Bit1
                                       ;    Bit0 FL Licht ein/aus
.def    Lowpegel      =    r20         ; Speicher fr Lowpegel
.def    Highpegel     =    r21         ; Speicher fr Highpegel
.def    w2            =    r22         ; Arbeitsregister 2
.def    eead          =    r23         ; eepromaddresse
.def    w3            =    r23         ; oder Arbeitsregister 3
.def    DimmerPWM     =    r24         ; PWM Zaehler fr LichtDimmer
.def    Zaehler       =    r25         ; Vorzhler zum ausfhren vom Lichtdimmer, zhlt die 10us ctc interrups
;.def    xl            =    r26         ;
;.def    xh            =    r27         ;
;.def    yl            =    r28         ; 16 bit Arbeits/Zeigerregister
;.def    yh            =    r29         ;
;.def    zl            =    r30         ; zum speichern der ADC Werte im Ram
;.def    zh            =    r31         ;

;********************************
;*        Daten def.            *
;********************************
.DSEG

EMK:                .byte 8    ; 8 Byte fr die letzten EMK Werte reservieren (ElektroMotorischeKraft)
LastRegStartVZ:     .byte 1    ; 1 Byte fr Lastreglungsstartverzgerung reservieren
AdresseLow:         .byte 1    ; 1 Byte fr lange Adresse LowByte reservieren
V_start:            .byte 1    ; 1 Byte fr V_start reservieren
IncWert:            .byte 1    ; 1 Byte fr Beschleunigungswert reservieren
DecWert:            .byte 1    ; 1 Byte fr Verzgerungswert reservieren
V_max:              .byte 1    ; 1 Byte fr V_max reservieren
LregSW:             .byte 1    ; 1 Byte fr Lastregelschrittweite reservieren
LregMax:            .byte 1    ; 1 Byte fr Max Lastregelweite reservieren
KickZeit:           .byte 1    ; 1 Byte fr Kickstartzeit reservieren
KickFS:             .byte 1    ; 1 Byte fr Kickstart Fahrstufe reservieren
Kennline:           .byte 28   ; 28 Byte fr Fahrstufen Kennline reservieren

;************************************************************************************************
;*                      ab hier Programmcode                                                    *
;************************************************************************************************
.CSEG

;************************************
;* Reset- und Interrupt-Vektoren    *
;************************************

.ORG $0000
        rjmp      RESET                ; Reset Handler
        reti;     EXT_INT0             ; IRQ0 Handler
        reti;     PCINT0               ; PCINT0 Handler
        reti;     PCINT1               ; PCINT1 Handler
        reti;     WATCHDOG             ; Watchdog Interrupt Handler
        reti;     TIM1_CAPT            ; Timer1 Capture Handler
        reti;     TIM1_COMPA           ; Timer1 CompareA Handler
        rjmp      TIM1_COMPB           ; Timer1 CompareB Handler
        reti;     TIM1_OVF             ; Timer1 Overflow Handler
        rjmp      TIM0_COMPA           ; Timer0 Compare A Handler
        reti;     TIM0_COMPB           ; Timer0 Compare B Handler
        reti;     TIM0_OVF             ; Timer0 Overflow Handler
        reti;     ANA_COMP             ; Analog Comparator Handler
        rjmp      ADC_Complete         ; ADC Conversion Handler
        reti;     EE_RDY               ; EEPROM Ready Handler
        reti;     USI_STR              ; USI STart Handler
        reti;     USI_OVF              ; USI Overflow Handler

;************************************************************************************************
;*                        Interruptprogramme                                                    *
;************************************************************************************************
;************************************
;*        Timer_0_interrupt         *
;************************************
; 1. Teil DCC Pin abtasten:
; wird alle 10us ausgelst und liest dann bit von int0/d2 (auch jeder andere pin mglich) ein
; dazu teiler auf1 (8tackte=1us) -> bis 80 zhlen lassen und einen Compare interrupt auslsen
; zw 26 und 34 cpu tackte -> 25-30% cpu leistung werden hier verbraucht!
; es werden die register intsreg,lowpegel,highpegel belegt
; das ergebnis wird in den flags bit und neuesbit zurckgegeben
; 2. Teil Lichtdimmer und Fahrstufen setzen
; wird alle 1,28ms zustzlich zu Teil 1 ausgefhrt und setzt die Fahrtrichtungsbeleuchtung in 10 Dimmstufen
; und danach je nach gewnschter beschleunigungs/verzgerungs/kickstartrate die die MotorPWM neu gesetzt
; dadurch verlngert sich der interrupt um bis zu 38 cpu tackte -> im schlechtesten Fall bentigt der inter-
; rupt fast 90% der cpu leistung, aus diesem Grund werden die register nur gesichert, wenn sie auch 
; verndert werden

TIM0_COMPA:
        in        intsreg,sreg         ; sreg sichern bewut nicht im fifo abgelegt, da so schneller

        sbic      pinb,2               ; wenn aktueller Pegel=0 1x springen
        inc       Highpegel            ; sonst Highpegel inc
        sbis      pinb,2               ; wenn aktueller Pegel=1 1x springen
        inc       Lowpegel             ; sonst Lowpegel inc

        cpi       Lowpegel,3           ; wenn mindestens 3x Low eingetroffen
        brne      PC+2                 ; dann 1x springe
        clr       Highpegel            ; sonst erst Highpegel lschen
        cpi       Highpegel,3          ; ist jetzt Highpegel = 3
        brne      DCC_abtasten_ENDE    ; wenn ungleich, dann fertig
        cbr       flag,(1<<bit)        ; neues bit lschen
        cpi       Lowpegel,7           ; ist Lowpegel kleiner 7
        brsh      PC+2                 ; wenn grer/gleich 1x springen
        sbr       flag,((1<<bit))      ; neues bit setzen
        sbr       flag,(1<<neuesbit)   ; flag neues bit da setzen
        clr       Lowpegel             ; Lowpegel wieder lschen
        inc       Highpegel            ; zur sicherheit 1x erhhen, damit nicht noch ein Byte erkannt wird
DCC_abtasten_ENDE:
                                       ; ab hier Licht Dimmer und Motor inc/dec setzen
        inc       Zaehler              ; dazu Zaehler register alle 10us inc
        cpi       Zaehler,128          ; wenn nicht 1x umgelaufen = 0 (10us+128=1,28ms vergangen)
        breq      PC+3
        out       sreg,intsreg         ; sreg wiederherstellen
        reti                           ; und fertig

        clr       Zaehler
Licht_dimmer:                          ; wird alle 1,28ms ausgefhrt
        sbrs      DCCreg,FL            ; wenn Licht an 1x springen
        rjmp      Licht_aus            ; sonst gleich zu...
        inc       DimmerPWM            ; inc
        cpi       DimmerPWM,11         ; auf max testen
        brne      Di_ausschalten       ; wenn nicht gehe zu..
        clr       DimmerPWM            ; wenn max, auf 0 setzen
        sbrs      DCCreg,Richtung      ; bei vorwrts 1x springen
        sbi       PortB,LichtR         ; Lichtausgang rckwrts einschalten
        sbrc      DCCreg,Richtung      ; wenn rckwrts 1x springen
        sbi       PortB,LichtV         ; Lichtausgang vorwrts einschalten
        rjmp      Motor                ; und weiter mit..
Di_ausschalten:
        cp        DimmerPWM,Dimmer     ; auf Dimmerwert testen
        brne      Motor                ; wenn ungleich gehe zu..
Licht_aus:
        cbi       PortB,LichtR         ; wenn gleich, Ausgnge abschalten
        cbi       PortB,LichtV
Motor:
        dec       FSzaehler            ; Fahrstufenzaehler dec
        tst       FSzaehler            ; auf Null testen
        breq      PC+3                 ; wenn gleich 3x springen
        out       sreg,intsreg         ; sreg wiederherstellen
        reti                           ; und fertig

        push      w                    ; w sichern
        push      w2                   ; w2 sichern
        cp        V_soll,V_ist         ; auf beschleunigen/bremsen testen
        Breq      V_ist_konst          ; wenn gleich gehe zu
        clr       EMK_soll             ; bei ungleich ersteinmal EMK_soll lschen, dann weiter verzweigen
        Brlo      V_bremsen            ; wenn V_soll kleiner gehe zu..
V_beschleunigen:                       ; wenn V_soll groer hier weiter
        tst       V_ist                ; ist V_ist=0 (Motor steht)?
        brne      kein_kick            ; wenn nicht  x springen
        lds       FSzaehler,KickZeit   ; Kickstartzeit setzen
        lds       w,KickFS             ; w mit Kickstart FS laden
        sbrc      DCCReg,nutzeKennl    ; wenn nicht nach Kennline in CV67-94 1xspringen
        rjmp      PC+2                 ; sonst mit V_start vorbelegen berspringen
        lds       V_ist,V_start        ; V_start holen und zur sicherheit auch noch
        inc       V_ist                ; V_ist 1x inc
        rjmp      FS_kick_setzen       ; und gleich ausgeben
kein_kick:
        lds       FSzaehler,IncWert    ; bescheunigungs wert laden
        inc       V_ist                ; V_ist erhhen
        rjmp      FS_setzen            ; und Fahrstufe setzen
V_bremsen:                             ; hier abbremsen
        lds       FSzaehler,DecWert    ; verzgerungs wert laden
        dec       V_ist                ; V_ist verringern
        rjmp      FS_setzen            ; und Fahrstufe setzen
V_ist_konst:                           ; Gleichbleibende Geschwindigkeit
        inc       FSzaehler            ; wieder auf1 -> in 2,56ms erneut hier um auf nderungen zu testen
        sbrc      Flag,kHzPWM          ; wenn MotorPWM nicht 16/32 kHz 1x springen
        sbi       ADCSRA, ADSC         ; bei 16/32kHz ADC Wandlung  zur Lastreglung starten
        rjmp      TIM2_ENDE            ; und fertig
FS_setzen:                             ; Timerregister mit neuer FS setzen
        mov       w,V_ist              ; V_ist nach w kopieren
FS_kick_setzen:                        ; hier landet Kickstarteinsprung
        sbrs      Flag,PWM9Bit         ; wenn 9 Bit PWM 1x springen
        rjmp      PC+5                 ; wenn nicht (8-bit PWM) 5x springen
        clr       w2                   ; w2 lschen
        lsl       w                    ; V_ist verdoppeln dazu *2 (aus 8 9 bit machen fr 16kHz )
        rol       w2                   ; bertrag in w2 einschieben
        out       OCR1AH,w2            ; high Byte (bit9) der PWM Impulsbreite setzen
        out       OCR1AL,w             ; PWM Impulsbreite setzen (bzw low Byte bei 9bit)
        in        w,TCCR1A
        sbr       w,(1<<COM1A1)        ; PWM Ausgang verbinden
        out       TCCR1A,w
TIM2_ENDE:
        pop       w2                   ; w2 wiederherstellen
        pop       w                    ; w wiederherstellen
        out       sreg,intsreg         ; sreg wiederherstellen
        reti                           ; und fertig

;************************************
;*        ADC Wert einlesen fertig  *
;************************************
; wenn (V_ist=V_soll) EMK Spannung einlesen und damit Motordrehzahl konstant halten
; ADC fertig wir ca 900 cpu Takte nach dem Start der Wandlung angesprungen
; interrupt verbraucht die ersten 7 Aufrufe 18 cpu Takte beim 8. Aufruf zw 40 und max 61 cpu Takte je
; nach Regelverhalten, liegt also unter den 80 cpu Takten des 10us ctc interrupt zum Abtasten des DCC
; Signals damit sollte keine Abtastung verloren gehen
; bei einer Motor-PWM-Frequenz von 16 oder 32kHz ist der Zeitpunkt der AD Wandlung egal, die PWM Impulse
; sind im Mewert mit integriert und stren nicht
; bei 30/120/500Hz PWM-Frequenz ist das nicht so, hier mu die Messung immer ca 900 cpu takte vor Beginn
; des PWM Impuls gestartet werden dann sind die Induktionsspitzen des letzten Impuls abgebaut und man mit
; nur noch die Generatorspannung
; -> 30Hz, PWM Teiler 1024, 8bit: Start der AD Wandlung bei TCNT1L = $FE (1*1024>900)
; ->120Hz, PWM Teiler 256,  8bit: Start der AD Wandlung bei TCNT1L = $FC (4*256 >900)
; ->500Hz, PWM Teiler 64,    8bit: Start der AD Wandlung bei TCNT1L = $F0 (15*64 >900)
; die maximale PWM-Impulsbreite ist 192 (von 256) Schritten -> die letzten 64 Schritte ist der PWM-Impuls
; immer aus 
 
ADC_Complete:
        in        intsreg,sreg         ; sreg sichern bewut nicht im fifo abgelegt, da so schneller
        push      w                    ; Arbeitsregister sichern
ADC_Werte_einlesen:                    ; und Mittelwert aus 8 Wandlungen ermitteln
;        in       w, ADCL              ; immer zuerst LOW Byte lesen (nicht ntig)
        in        w, ADCH              ; danach das mittlerweile gesperrte High Byte
                                       ; beides in w einlesen, da nur 8 bit ntig
        st        z+,w                 ; wert im ram ablegen
        cpi       zl,low(EMK)+8        ; 8 wandlungen duchgefhrt?
        breq      PC+2                 ; wenn ja 1x springen
        rjmp      ADC_ende             ; wenn nicht gehe zu..
mittelwert:
        clr       yh                   ; ansonsten neuen mittelwert errechnen
        ld        yl,-z
        ld        w,-z
        add       yl,w                 ; w zu y pointer addieren (zh ist null -> nur
        adc       yh,zh                ; der bertrag wird zu yh addiert
        cpi       zl,low(EMK)          ; alle werte addiert?
        brne      PC-4                 ; wenn nicht weitermachen
        ror       yh                   ; yp 3x nach rechts verschieben (durch 8 teilen)
        ror       yl
        ror       yh
        ror       yl
        ror       yh
        ror       yl                   ; hier ist der neue mittelwert

        tst       EMK_soll             ; EMK_soll auf 0 testen
        brne      Lastreglung          ; wenn ungleich gehe zu..
        sbrs      Flag,kHzPWM          ; wenn PWM freq 16/32 kHz 1x springen
        rjmp      Speichere_Sollwert   ; bei 30-500Hz Startverzgerung berspringen
        lds       w,LastRegStartVZ     ; Vergleich auf ablauf der Startverzgerung
        inc       w                    ; gibt der motordrehzahl nach erreichen von
        sts       LastRegStartVZ,w     ; V_ist=V_soll Zeit sich einzupegeln
        cpi       w,100                ; es werden 100*8= 800 Messungen verworfen
        brne      ADC_ende             ; und somit hier gleich zum Ende gesprungen
Speichere_Sollwert:
        mov       EMK_soll,yl          ; die nchsten werden dann als Sollwert gespeichert
        clr       EMK_pos              ; die regelposition auf 0 schritte Abweichung
        rjmp      ADC_ende
Lastreglung:                           ; Sollwert ist vorhanden
        clr       w                    ; Variable lschen bei 30-500 Hz umsonst
        sts       LastRegStartVZ,w     ; (berspringen wrde aber auch 2 Befehle kosten)
        cp        EMK_soll,yl          ; neuen Mewert mit sollwert vergleichen
        Breq      ADC_ende             ; wenn gleich nichts machen
        Brlo      Drehzahl_zu_hoch     ; wenn Sollwert kleiner Mewert, gehe zu..
Drehzahl_zu_klein:                     ; Sollwert ist grer Mewert
        mov       w,EMK_soll           ; testen ob mindestabweichung vorliegt
        sub       w,yl                 ; dazu Mewert vom soll abziehen
        cp        w,LregMin            ; und mit Mindestabweichung vergleichen
        brlo      ADC_ende             ; wenn nicht, nicht nachregeln
        in        yl,OCR1AL            ; aktuell gesetzen PWM Wert holen
        in        yh,OCR1AH
        lds       w,LregMax            ; w mit max regelweite laden
        cp        EMK_pos,w            ; auf max regelweite testen
        brge      ADC_ende             ; wenn vorzeichenbehaftet w grer oder gleich EMK_pos, nichts machen
        lds       w,LregSW             ; ansonsten Lastregelschrittweite holen und
        add       EMK_pos,w            ; 1.neue Regelposition setzen
        clc                            ; c lschen
        sbrc      Flag,PWM9bit         ; wenn nicht auf 16kHz/9bit PWM steht 1x springen
        rol       w                    ; sonst doppelte schrittweite w=w*2
        add       yl,w                 ; 2.regelschrittweite addieren
        clr       w                    ; (9bit ntig darum noch c flag zu yh addieren)
        adc       yh,w
        sbrs      Flag,PWM9bit         ; auf 16kHz/9bit PWM testen
        rjmp      PC+4                 ; wenn 8bit, springe zu..
        cpi       yh,2                 ; yh max 1 mglich
        brsh      ADC_ende             ; wenn grer oder gleich gehe zu..
        rjmp      PWM_neusetzen        ; sonst korregierten wert setzen
        tst       yh                   ; bei 8bit mu yh 0 sein
        brne      ADC_ende             ; wenn ungleich gehe zu..
        rjmp      PWM_neusetzen        ; sonst korregierten wert setzen
Drehzahl_zu_hoch:                      ; Sollwert ist kleiner Mewert
        sub       yl,EMK_soll          ; testen ob mindestabweichung vorliegt, dazu soll vom mewert abziehen
        cp        yl,LregMin           ; und mit Mindestabweichung vergleichen
        brlo      ADC_ende             ; wenn nicht, nicht nachregeln
        in        yl,OCR1AL            ; aktuell gesetzen PWM Wert holen
        in        yh,OCR1AH
        lds       w,LregMax            ; w mit max regelweite laden
        com       w                    ; auf max Regelweite testen
        cp        EMK_pos,w
        Brlt      ADC_ende             ; wenn vorzeichbehaftet w grer EMK_pos, nichts machen
        lds       w,LregSW             ; sonst Lastregelschrittweite holen und
        sub       EMK_pos,w            ; 1. neue regelposition setzen
        clc                            ; c lschen
        sbrc      Flag,PWM9bit         ; wenn nicht auf 16kHz/9bit PWM steht 1x springen
        rol       w                    ; sonst doppelte schrittweite w=w*2
        sub       yl,w                 ; 2. regelschrittweite subtrahieren
        clr       w
        sbc       yh,w
        brmi      ADC_ende             ; wenn yh jetzt negativ (kleiner 0) nichts mehr machen
PWM_neusetzen:                         ; Werte neu setzen
        out       OCR1AH,yh
        out       OCR1AL,yl
ADC_ende:
        pop       w                    ; w wiederherstellen
        out       sreg,intsreg         ; sreg wiederherstellen
        reti

;************************************
;*        Timer1 CompareB interupt  *
;************************************
; wird beim Zhlerstand $F0 ausgelst und dient nur zum starten des AD Wandler bei 30-500Hz PWM
; der interrupt wir beim auch nur dann angesprungen

TIM1_COMPB:
        sbi       ADCSRA, ADSC         ; ADC Wandlung  zur Lastreglung starten
        reti

;************************************************************************************************
;*                        Unterprogramme                                                        *
;************************************************************************************************
.include "init_t24lokdec.inc"          ; init des tiny24
.include "DCC.inc"                     ; DCC Informationen einlesen
.include "Eprom.inc"                   ; liest oder schreibt ein Byte aus eead in w
.include "Funktionsbefehle.inc"        ; setzt die Funktionsausgnge
.include "Fahrbefehle.inc"             ; setzt die Fahrbefehle (Geschwindigkeit + Richtung)
.include "Programmierung.inc"          ; Programmiert den Dekoder
.include "LoadEEdaten.inc"             ; DekoderDaten aus EEProm einlesen


;************************************************************************************************
;*                        Hauptprogramm                                                         *
;************************************************************************************************

DCCreset:
        clr       Flag                 ; wenn Reset ber DCC Befehl, dann Flag setzen
        sbr       Flag,(1<<resetbit)
        rjmp      PC+2                 ; und 1x springen
Reset:  clr       Flag                 ; wenn Reset nicht per Befehl, kein Flag setzen
;        ldi      w,high(RAMEND)       ; StackPointer setzen
;        out      SPH,w
        ldi       w,low(RAMEND)
        out       SPL,w
        cli                            ; interrups ausschalten
; Ports setzen
        ldi       w,0b11111110         ; PA1 Eingang rest Ausgang
        out       DDRA,w
        ldi       w,0b00000011         ; PB0-1 Ausgang rest Eingang
        out       DDRB,w
        clr       w                    ; alle Ports ausschalten
        out       Porta,w
        out       Portb,w
;        sbi      Portb,2              ; Pullup DCC Pin ein
; register laden
;        clr      w                    ; EEProm high Addresse lschen
;        out      EEARH,w
        clr       V_ist
        clr       V_soll
        clr       EMK_soll
        clr       lowpegel
        clr       highpegel
        clr       Zaehler
        clr       FSzaehler
        inc       FSzaehler
        ldi       zl,LOW(EMK)          ; den Z-Pointer mit dem Start der Generator Bytes laden
        ldi       zh,HIGH(EMK)
        rcall     EEpromDatenlesen     ; DekoderDaten aus EEProm einlesen
; Hardware init
        rcall     init_adc_t24         ; ADC init
        rcall     init_timer1_t24      ; Timer1 init -> Motor PWM Ansteuerung
        rcall     init_timer0_t24      ; Timer0 init -> 10us CTC interrupt zum DCC Signal einlesen

        sei                            ; interrups einschalten

; ab hier normaler Dekoderschleife
Hauptschleife:
        rcall     DCC_schleife         ; auf neue DCC Daten warten
                                       ; neue Daten prfen und bergabe
        sbrc      Flag,resetbit        ; wenn 1 war reset da gehe zu
        rjmp      ServiceMode          ; Programmiermodus CV-Lesen, Schreiben, Bitmanipulation
                                       ; prfe Adresse auf 0
        tst       byteA                ; wenn 0 dann
        breq      an_alle              ; Broadcast an alle

        ldi       w,$FF                ; prfe Adresse auf $FF
        cp        byteA,w              ;
        brne      PC+2                 ; wenn ungleich 1x springen
Leerlauf:                              ; wenn gleich Leerlaufpaket -> nichts machen bei diesem befehl)
        rjmp      Hauptschleife

        cp        byteA,Adresse        ; wenn eigene Adresse
        breq      an_mich              ; dann test OK

        rjmp      Hauptschleife        ; sonst Hauptschleife

an_mich:                               ; eigene Adresse ist angesprochen
        sbrs      DCCreg,erwAdresse    ; hier erst noch auf lange Adresse testen
        rjmp      an_alle              ; wenn nicht gleich weiter
        lds       w,AdresseLow         ; sonst erst noch auf richtiges LowAdressbyte testen
        cp        byteB,w
        brne      Hauptschleife        ; wenn nicht gleich gehe zu..
        mov       byteB,byteC          ; wenn gleich alle datenBytes an die position
        mov       byteC,byteD          ; fr 7bit addressen verschieben
        mov       byteD,byteE
        mov       byteE,byteF
an_alle:                               ; alle Decoder sind angesprochen
;hier je nach Datenformat verzweigen
        mov       w,byteB              ; Daten sichern
        cpi       w,0b00000000         ; Vergleich auf dcc reset
        brne      PC+2                 ; wenn ungleich 1x springen
        rjmp      DCCreset             ; sonst Reset machen
        andi      w,0b11100000         ; Datenbits ausblenden
        cpi       w,0b00100000         ; Vergleiche auf erweiterte Operation (enthlt befehl fr 128 Fahrstufen)
        brne      PC+4                 ; wenn ungleich 4x springen
        mov       w2,byteB             ; w2 mit inst Byte laden
        mov       w,byteC              ; w mit Daten laden
        rjmp      erw_Operation        ; wenn gleich gehe zu..
        cpi       w,0b01000000         ; Vergleiche mit V Rckwrts einfaches Format
        brne      PC+3                 ; wenn ungleich 2x springen
        mov       w,byteB              ; w mit Daten laden
        rjmp      V_Basis              ; wenn ja gehe zu
        cpi       w,0b01100000         ; Vergleiche mit V Vorwrts einfaches Format
        brne      PC+3                 ; wenn ungleich 2x springen
        mov       w,byteB              ; w mit Daten laden
        rjmp      V_Basis              ; wenn ja gehe zu..
        cpi       w,0b10000000         ; Vergleiche mit Funktionsgruppe F1-F4
        brne      PC+3                 ; wenn ungleich 2x springen
        mov       w,byteB              ; w mit Funktionsdaten laden
        rjmp      Fgruppe1             ; und dann gehe zu..
        cpi       w,0b11100000         ; Vergleich auf CV reg Manipulation
        brne      PC+2                 ; wenn ungleich 1x springen
        rjmp      CV_edit              ; wenn gleich gehe zu..
                                       ; alles andere wird nicht unterstzt -> verwerfen und zurck
        rjmp      Hauptschleife        ; Endlosschleife

.ESEG

;CV Register im eeprom vom Lokdec

.db 1             ;CV1 Dekoderaddresse 1-127
.db 0             ;CV2 Minimale Geschwindigkeit 0-63
.db 40            ;CV3 Anfahrverzgerung 1-256 1=1,28ms je Schritt 0(256)=327,68ms je Schritt
.db 40            ;CV4 Bremsverzgerung 1-256 w.o.
.db 150           ;CV5 Maximale Geschwindigkeit in 6 Stufen: 25% 50% 75% 100% 125% 150%
.db $FF           ;CV6
.db 1             ;CV7 Software-Version
.db $0D           ;CV8 Herstellerkennung
.db 3             ;CV9 Motor PWM-Periode 0-4 0=30Hz 1=120Hz 2=500Hz 3=16KHz 4=32KHz
.db $FF           ;CV10
.db $FF           ;CV11
.db $FF           ;CV12
.db $FF           ;CV13
.db $FF           ;CV14
.db $FF           ;CV15
.db $FF           ;CV16
.db 192           ;CV17 Erweiterte Adresse Hherwertiges Byte der langen Adresse plus 192
.db 100           ;CV18 Erweiterte Adresse Niederwertiges Byte der langen Adresse
.db $FF           ;CV19 Consist-Adresse wird nicht untersttzt!
.db $FF           ;CV20
.db $FF           ;CV21
.db $FF           ;CV22
.db $FF           ;CV23
.db $FF           ;CV24
.db $FF           ;CV25
.db $FF           ;CV26
.db $FF           ;CV27
.db $FF           ;CV28
.db 2             ;CV29 Konfiguration nach DCC-Norm
                  ;        +1:Fahr-Richtung umgekehrt
                  ;        +2:28/128 Fahrstufen, sonst 14
                  ;        +4:automatisch Analog/Digital-Erkennung, sonst nur Digitalbetrieb
                  ;        +8:letzte Geschwindigkeit+Richtung im EEprom merken(gibt es nicht nach Norm)
                  ;        +16:Kennlinie auf CV66-95, sonst Fahrstufen ber CV2, CV5 und CV6
                  ;        +32:Erweiterte (lange) Adresse (CV17 und CV18), sonst CV1
                  ;        +128:Zubehrdekoder, sonst Lokdekoder
;CV30-48
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db $FF           ;CV49 - CV64 Herstellerparameter
.db 5             ;CV50 Dimmen der Lichtausgnge 0-10
.db 0             ;CV51 Vmerken hier letzte Geschwindigkeit+Richtung merken
.db 2             ;CV52 Schrittweite der Lastreglung 0=aus Wert liegt zw 1 und ca 10 max 63
.db 127           ;CV53 Max Anzahl der Korrekturschritte (aus CV52) zur Lastreglung 0 bis 127 (128 Stufen)
.db 3             ;CV54 minimale Abweichung vom Sollwert bevor Korrektur beginnt von 0 bis <CV53
;CV55-64
.db $FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF
.db 20            ;CV65 Kickstartzeit 1-256 1=1,28ms 0(256)=327,68ms lang V_laut_CV66 beim anfahren
.db $7F           ;CV66 KickstartSpannung 0=aus 255=Vmax
.db 10            ;CV67 - 94
.db 20            ; alternative Geschwindigkeitkennlinie. (laut Norm nur bei Modus 28 Fahrstufen)
.db 28            ; Jeder der 28 Fahrstufen wird eine spezifische Motorspannung zugeordnet
.db 36            ; mindestwert ist 0 (Motor aus) und 255 maximale Motorspannung
.db 44            ; (hier auch bei 14 Fahrstufen untersttzt - es wird jeder 2. Wert benutzt)
.db 48            ;CV74 Fahrstufe 06
.db 56            ;CV74 Fahrstufe 07
.db 64            ;CV74 Fahrstufe 08
.db 72            ;CV75 Fahrstufe 09
.db 80            ;CV76 Fahrstufe 10
.db 88            ;CV77 Fahrstufe 11
.db 96            ;CV78 Fahrstufe 12
.db 104           ;CV79 Fahrstufe 13
.db 112           ;CV80 Fahrstufe 14
.db 120           ;CV81 Fahrstufe 15
.db 128           ;CV82 Fahrstufe 16
.db 136           ;CV83 Fahrstufe 17
.db 144           ;CV84 Fahrstufe 18
.db 152           ;CV85 Fahrstufe 19
.db 160           ;CV86 Fahrstufe 20
.db 168           ;CV87 Fahrstufe 21
.db 176           ;CV88 Fahrstufe 22
.db 184           ;CV89 Fahrstufe 23
.db 192           ;CV90 Fahrstufe 24
.db 200           ;CV91 Fahrstufe 25
.db 208           ;CV92 Fahrstufe 26
.db 216           ;CV93 Fahrstufe 27
.db 224           ;CV94 Fahrstufe 28
.db $FF,$FF,$FF,$FF,$FF,$FF
.db $A3           ; Speicherplatz OSCCAL
