;************************************************************************************************
;* init der tiny24 Hardware fr dcc lokdecoder                                                  *
;************************************************************************************************

;********************************
;*        t24 Timer0 init       *
;********************************
init_timer0_t24:

;*    Timer0 auf 10us CTC Interrupt setzen    *
; Timer/Counter Control Register A
        ldi       w,0b00000010         ; +------+------+------+------+------+------+------+------+
        out       TCCR0A,w             ; |COM0A1|COM0A0|COM0B1|COM0B0|   -  |   -  | WGM01| WGM00|
                                       ; +------+------+------+------+------+------+------+------+
                                       ; Bits 7/6 COM0A1/0: Compare Match Output A Mode
                                       ; Bits 5/4 COM0B1/0: Compare Match Output B Mode
                                       ; Bits 1/0 WGM01/0: Waveform Generation Mode
                                       ; Mode WGM2 WGM1 WGM0    Mode         TOP     Update OCRx     TOV set1
                                       ;    0    0    0     0         Normal         0xFF     Immediate         MAX
                                       ;    1    0    0     1        PWM PhCo    0xFF     TOP             BOTTOM
                                       ;    2    0    1     0        CTC         OCRA     Immediate         MAX
                                       ;    3     0     1      1         Fast PWM     0xFF     BOTTOM             MAX
                                       ;    4     1     0      0         Reserved
                                       ;    5     1     0      1        PWM PhCo    OCRA     TOP             BOTTOM
                                       ;    6     1     1      0         Reserved
                                       ;    7     1     1      1         Fast PWM     OCRA     BOTTOM             TOP

; Timer/Counter0 Control Register B
; timer mit Teiler auf ck/1 starten
        ldi       w,0b00000001         ; +------+------+------+------+------+------+------+------+
        out       TCCR0B,w             ; |FOC0A |FOC0B |  -   |   -  |WGM02 | CS02 | CS01 | CS00 |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; Bit 7 FOC0A: Force Output Compare A
                                       ; Bit 6 FOC0B: Force Output Compare B
                                       ; Bit WGM02: Waveform Generation Mode
                                       ; Bits 2-0 CS02-0: Clock Select
                                       ; CS02 CS01 CS00
                                       ;    0    0    0     No clock source (Timer/Counter stopped)
                                       ;     0     0     1     clkI/O/(No prescaling)
                                       ;     0     1     0     clkI/O/8 (From prescaler)
                                       ;     0     1     1     clkI/O/64 (From prescaler)
                                       ;     1     0     0     clkI/O/256 (From prescaler)
                                       ;     1     0     1     clkI/O/1024 (From prescaler)
                                       ;     1     1     0     External clock on T0 pin. Clock on falling edge.
                                       ;     1     1     1     External clock on T0 pin. Clock on rising edge.

; Timer/Counter0 Register
; Zhler setzen
        clr       w
        out       tcnt0,w

; Output Compare Register              ; mit 80 Tackten laden
        ldi       w,80
        out       OCR0A,w

; Output Compare Register              ; wird nicht gebraucht
;        clr      w
;        out      OCR0B,w

; Timer/Counter0 Interrupt Mask Register TIMSK0:
        ldi       w,0b00000010         ; +------+------+------+------+------+------+------+------+
        out       TIMSK0,w             ; |  -   |   -  |   -  |   -  |   -  |OCIE0B|OCIE0A| TOIE0|
                                       ; +------+------+------+------+------+------+------+------+
                                       ; OCIE0B: Timer/Counter0, Output Compare B Match Interrupt Enable
                                       ; OCIE0A: Timer/Counter0, Output Compare A Match Interrupt Enable
                                       ; TOIE0: Timer/Counter0, Overflow Interrupt Enable

; Timer/Counter0 Interrupt Flag Register TIFR0: lsche Timer0 Interupt
        ldi       w,0b00000000         ; +------+------+------+------+------+------+------+------+
        out       TIFR0,w              ; |   -  |   -  |   -  |   -  |   -  |OCF0B |OCF0A | TOV0 |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; OCF0B: Timer/Counter, Output Compare B Match Flag
                                       ; OCF0A: Timer/Counter0, Output Compare A Match Flag
                                       ; TOV0: Timer/Counter0, Overflow Flag
        ret

;********************************
;*        Timer1 init           *
;********************************
; Timer1 initialisieren:

init_timer1_t24:
; Timer/Counter1 Control Register TCCR1A -> 8bit fast pwm mode5 Ausgang noch getrennt
;*    wird beim EEpromload passend gesetzt    *
;        ldi       w,0b00000000        ; +------+------+------+------+------+------+------+------+
;        out       TCCR1A,w            ; |COM1A1|COM1A0|COM1B1|COM1B0|  -   |  -   |WGM11 |WGM10 |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; COM1A1/0: Compare Output Mode for channel A
                                       ; COM1B1/0: Compare Output Mode for channel B
                                       ;  0 0 Normal port operation, OC1A/OC1B disconnected
                                       ;  1 0 Clear OC1A/OC1B on Compare Match, set OC1A/OC1B at TOP(fast PWM Mode)
                                       ;  1 1 Set OC1A/OC1B on Compare Match, clear OC1A/OC1B at TOP(fast PWM Mode)
                                       ; FOC1A/B: Force Output Compare for channel A/B
                                       ;  0 0 Normal port operation
                                       ; WGM11/0: Waveform Generation Mode
                                       ;  Mode WGM13  WGM12  WGM11  WGM10  Timer/Counter TOP    Update    TOV1
                                       ;             (CTC1) (PWM11)(PWM10) Mode          OCR1x
                                       ;   0      0      0      0      0   Normal        0xFFFF Immediate MAX
                                       ;   1      0      0      0      1   PWM PhCo8-bit 0x00FF   TOP     BOTTOM
                                       ;   2      0      0      1      0   PWM PhCo9-bit 0x01FF   TOP     BOTTOM
                                       ;   3      0      0      1      1   PWM PhCo10bit 0x03FF   TOP     BOTTOM
                                       ;   4      0      1      0      0   CTC           OCR1A  Immediate MAX
                                       ;   5      0      1      0      1   Fast PWM8-bit 0x00FF   TOP     TOP
                                       ;   6      0      1      1      0   Fast PWM9-bit 0x01FF   TOP     TOP
                                       ;   7      0      1      1      1   Fast PWM10bit 0x03FF   TOP     TOP
                                       ;   8      1      0      0      0   PWM PhFrCo    ICR1    BOTTOM   BOTTOM
                                       ;   9      1      0      0      1   PWM PhFrCo    OCR1A   BOTTOM   BOTTOM
                                       ;   10     1      0      1      0   PWM PhCorrect ICR1     TOP     BOTTOM
                                       ;   11     1      0      1      1   PWM PhCorrect OCR1A    TOP     BOTTOM
                                       ;   12     1      1      0      0   CTC           ICR1   Immediate MAX
                                       ;   13     1      1      0      1   (Reserved)
                                       ;   14     1      1      1      0   Fast PWM      ICR1     TOP     TOP
                                       ;   15     1      1      1      1   Fast PWM      OCR1A    TOP     TOP

; Timer/Counter1 Control Register  TCCR1B
;*    wird beim EEPromload gesetzt    *
;        ldi       w,0b00001101        ; +------+------+------+------+------+------+------+------+
;        out       TCCR1B,w            ; | ICNC1| ICES1|      | WGM13| WGM12| CS12 | CS11 | CS10 |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; CS12 - CS10   Clock SelectCS12    CS11    CS10    Bedeutung
                                       ;    0    0    0    keine (Der Timer ist angehalten)
                                       ;    0    0    1    Vorteiler: 1
                                       ;    0    1    0    Vorteiler: 8
                                       ;    0    1    1    Vorteiler: 64
                                       ;    1    0    0    Vorteiler: 256
                                       ;    1    0    1    Vorteiler: 1024
                                       ;    1    1    0    Externer Takt vom Pin T1, fallende Flanke
                                       ;    1    1    1    Externer Takt vom Pin T1, steigende Flanke
                                       ; WGM13:12: Waveform Generation Mode siehe TCCR1A
                                       ; ICES1   Input Capture Edge Select
                                       ; ICNC1   Input Capture Noise Canceler

; Timer/Counter1 Control Register  TCCR1C
        ldi       w,0b00000000         ; +------+------+------+------+------+------+------+------+
        out       TCCR1C,w             ; | FOC1A| FOC1B|      |      |      |      |      |      |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; Bit 7 FOC1A: Force Output Compare for Channel A
                                       ; Bit 6 FOC1B: Force Output Compare for Channel B

; Timer/Counter 1 TCNT1H and TCNT1L
        clr       w
        out       TCNT1H,w
        out       TCNT1L,w

; Output Compare Register 1 A OCR1AH and OCR1AL
        clr       w
        out       OCR1AH,w
        out       OCR1AL,w

; Output Compare Register 1 B OCR1BH and OCR1BL
; OCR1BL auf Zeitpunkt fr Beginn ADC Wandlung fr 30-500Hz Motor PWM setzen (wird immer gesetzt) 
        clr       w
        out       OCR1BH,w
        ldi       w,$F0
        out       OCR1BL,w

; Input Capture Register 1 ICR1H and ICR1L
        clr       w
        out       ICR1H,w
        out       ICR1L,w

; Timer/Counter Interrupt Mask Register TIMSK1:
        clr       w                    ;
        sbrs      Flag,kHzPWM          ; bei 16/32 kHz nicht setzen (1x springen)
        sbr       w,(1<<OCIE1B)        ; +------+------+------+------+------+------+------+------+
        out       TIMSK1,w             ; |      |      | ICIE1|      |      |OCIE1B|OCIE1A| TOIE1|
                                       ; +------+------+------+------+------+------+------+------+
                                       ; ICIE1: Timer/Counter1, Input Capture Interrupt Enable
                                       ; OCIE1A: Timer/Counter1, Output Compare A Match Interrupt Enable
                                       ; OCIE1B: Timer/Counter1, Output Compare B Match Interrupt Enable
                                       ; TOIE1: Timer/Counter1, Overflow Interrupt Enable

; Timer/Counter Interrupt Flag Register TIFR1:
; wird immer gelscht, auch wenn nicht verwendet 
        clr       w                    ; +------+------+------+------+------+------+------+------+
        out       TIFR1,w              ; |      |      | ICF1 |      |      |OCF1B |OCF1A | TOV1 |
                                       ; +------+------+------+------+------+------+------+------+
                                       ; ICF1: Timer/Counter1, Input Capture Flag
                                       ; OCF1A: Timer/Counter1, Output Compare A Match Flag
                                       ; OCF1B: Timer/Counter1, Output Compare B Match Flag
                                       ; TOV1: Timer/Counter1, Overflow Flag
        ret


;********************************
;*        ADC init              *
;********************************
; ADC initialisieren: Single Conversion, Vorteiler 64, interrupt, Vref=AVCC, linksbndig, PC0
 
init_adc_t24:
        ldi       w,0b00000000         ;  ADMUX:
        out       ADMUX, w             ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; | REFS1 | REFS0 | MUX5  | MUX4  |  MUX3 |  MUX2 |  MUX1 |  MUX0 |
                                       ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; Referenzspannung REFS1, REFS0
                                       ; 0 0 VCC used as analog reference, disconnected from PA0 (AREF).
                                       ; 0 1 External Voltage Reference at PA0 (AREF) pin, Int Volt Ref turned off.
                                       ; 1 0 Internal 1.1V Voltage Reference.
                                       ; 1 1 Reserved.
                                       ; MUX5 MUX4 MUX3 MUX2 MUX1 MUX0
                                       ;    0     0      0       0    0     0    Kanal 0, Pin PA0
                                       ;    0     0      0       0    0     1    Kanal 1, Pin PA1
                                       ;    0     0      0       0    1     0    Kanal 2, Pin PA2
                                       ;    0     0      0       0    1    1    Kanal 3, Pin PA3
                                       ;    0     0      0       1    0     0    Kanal 4, Pin PA4
                                       ;    0     0      0       1    0     1    Kanal 5, Pin PA5
                                       ;    0     0      0       1    1     0    Kanal 6, Pin PA6
                                       ;    0     0      0       1    1    1    Kanal 7, Pin PA7


; ADCSRA ADC Control and Status Register A
        ldi       w,0b10001110         ;  ADCSRA:
        out       ADCSRA, w            ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; |  ADEN |  ADSC | ADATE |  ADIF |  ADIE | ADPS2 | ADPS1 | ADPS0 |
                                       ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; ADEN (ADC Enable): Mittels ADEN wird der ADC ein und ausgeschaltet.
                                       ;    Eine 1 an dieser Bitposition schaltet den ADC ein.
                                       ; ADSC (ADC Start Conversion): Wird eine 1 an diese Bitposition geschrieben,
                                       ;    so beginnt der ADC mit der Wandlung. Das Bit bleibt auf 1, solange die
                                       ;    Wandlung im Gange ist.
                                       ;    Wenn die Wandlung beendet ist, wird dieses Bit von der ADC Hardware
                                       ;    wieder auf 0 gesetzt.
                                       ; ADATE: ADC Auto Trigger Enable: Wird eine 1 an ADATE geschrieben,
                                       ;    so wird der ADC im Auto Trigger Modus betrieben. Dabei startet der ADC
                                       ;    beim auslsen
                                       ;    einer Triggerquelle automatisch die nchste Messung.
                                       ; ADIF (ADC Interrupt Flag): Wenn eine Messung abgeschlossen ist,
                                       ;    wird das ADIF Bit gesetzt. Ist zustzlich noch das ADIE Bit gesetzt,
                                       ;    so wird ein Interrupt ausgelst und der entsprechende Interrupt Handler
                                       ;    angesprungen.
                                       ; ADIE (ADC Interrupt Enable): Wird eine 1 an ADIE geschrieben,
                                       ;    so lst der ADC nach Beendigung einer Messung einen Interrupt aus.
                                       ; ADPS2, ADPS1, ADPS0 (ADC Prescaler): Mit dem Prescaler kann die ADC-Frequenz
                                       ;    gewhlt werden.
                                       ;    Laut Datenblatt sollte diese fr die optimale Auflsung zwischen 50KHz
                                       ;    und 200kHz liegen.
                                       ;    Ist die Wandlerfrequenz langsamer eingestellt, kann es passieren dass die
                                       ;    eingebaute Sample & Hold
                                       ;    Schaltung die Eingangsspannung nicht lange genug konstant halten kann.
                                       ;    Ist die Frequenz aber zu schnell
                                       ;    eingestellt, dann kann es passieren dass sich die Sample & Hold Schaltung
                                       ;    nicht schnell genug an die
                                       ;    Eingangsspannung anpassen kann.
                                       ;    ADPS2    ADPS1    ADPS0    Vorteiler
                                       ;    0    0    0    2
                                       ;    0    0    1    2
                                       ;    0    1    0    4
                                       ;    0    1    1    8
                                       ;    1    0    0    16
                                       ;    1    0    1    32
                                       ;    1    1    0    64
                                       ;    1    1    1    128

; ADCSRB ADC Control and Status Register B
        ldi       w,0b00010000         ;  ADCSRB:
        out       ADCSRB, w            ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; |  BIN  |  ACME |       | ADLAR |       | ADTS2 | ADTS1 | ADTS0 |
                                       ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; BIN: Bipolar Input Mode
                                       ; ACME: Analog Comparator Multiplexer Enable
                                       ; Ausrichtung ADLAR
                                       ; 0    Das Ergebnis wird in den Registern ADCH/ADCL rechtsbndig ausgerichtet.
                                       ;    Die 8 niederwertigsten Bits des Ergebnisses werden in ADCL abgelegt.
                                       ;    Die verbleibenden 2 Bits des Ergebnisses werden im Register ADCH in den
                                       ;    Bits 0 und 1 abgelegt.
                                       ; 1    Das Ergebnis wird in den Registern ADCH/ADCL linksbndig ausgerichtet.
                                       ;    Die 8 hchstwertigen Bits des Ergebnisses werden in ADCH abgelegt.
                                       ;    Die verbleibenden 2 niederwertigen Bits werden im Register ADCL in den
                                       ;    Bits 6 und 7 abgelegt.
                                       ; ADTS2-0: ADC Auto Trigger Source

;DIDR0 Digital Input Disable Register 0
        ldi       w,0b00000001         ;  DIDR0
        out       DIDR0, w             ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; | ADC7D | ADC6D | ADC5D | ADC4D | ADC3D | ADC2D | ADC1D | ADC0D |
                                       ; +-------+-------+-------+-------+-------+-------+-------+-------+
                                       ; ADC7D..ADC0D: ADC7..0 Digital Input Disable
        ret
